import { AfterViewInit, Component, ContentChildren, Directive, ElementRef, Input, OnInit, QueryList, TemplateRef, ViewChild, ViewChildren } from '@angular/core';
import { CarouselDirective } from './../../directives/carousel.directive';
import { animate, AnimationBuilder, AnimationFactory, AnimationPlayer, style } from '@angular/animations';

@Directive({
  selector: '.carousel-item'
})
export class CarouselItemElement {
}

@Component({
  selector: 'carousel',
  exportAs: 'carousel',
  template: `
    <section class="carousel-wrapper" [ngStyle]="carouselWrapperStyle">
      <ul class="carousel-inner" #carousel>
        <li *ngFor="let item of items;" class="carousel-item">
          <ng-container [ngTemplateOutlet]="item.tpl"></ng-container>
        </li>
      </ul>
    </section>
    <div *ngIf="showControls" style="margin-top: 1em">
      <button (click)="next()" class="btn btn-default">Next</button>
      <button (click)="prev()" class="btn btn-default">Prev</button>
    </div>
  `,
  styles: [`
   .offer_slider {
    padding: 10px;
    padding-top: 0px;
    .slick-slide {
        margin: 0 5px;
    }
    .slide {
        position: relative;
        padding-bottom: 10px;
        .inner_slider {
            height: 160px;
            width: 100%;
            .content_inner {
                position: relative;
                top: 10px;
                left: 15px;
            }
            background-image: url("../../../assets/offer/offer_temp.png");
            background-position: center;
            background-repeat: no-repeat;
            background-size: cover;
            border-radius: 10px;
            border-bottom-right-radius: 0px;
            position: relative;
            .content_inner {
                h3 {
                    margin: 0px;
                    padding: 0px;
                    color: #fff;
                    padding-top: 10px;
                    padding-bottom: 10px;
                }
                p {
                    margin: 0px;
                    padding: 0px;
                    color: #fff;
                    font-size: 13px;
                    font-weight: 300;
                }
            }
            .coupon_code {
                border: 1px dashed #fff;
                background-size: 9px;
                position: absolute;
                bottom: 10px;
                right: 10px;
                border-radius: 3px;
                background-image: url("../../../assets/offer/copy.png");
                background-position: right 10px top 10px;
                background-repeat: no-repeat;
                padding-right: 30px;
                font-size: 11px;
                .code {
                    color: #fff;
                    padding: 8px;
                    border-right: 1px dashed #fff;
                }
                .copy {}
            }
        }
        img {
            position: absolute;
            bottom: 7px;
            right: 0px;
            left: 55px;
            width: calc(100% - 55px);
        }
    }
}

    .carousel-wrapper {
      overflow: hidden;
    }

    .carousel-inner {
      display: flex;
    }

  `]
})
export class CarouselComponent implements AfterViewInit {
    @ContentChildren(CarouselDirective) items: QueryList<CarouselDirective>;
  @ViewChildren(CarouselItemElement, { read: ElementRef }) private itemsElements: QueryList<ElementRef>;
  @ViewChild('carousel', {static: false}) private carousel: ElementRef;
  @Input() timing = '250ms ease-in';
  @Input() showControls = true;
  private player: AnimationPlayer;
  private itemWidth: number;
  private currentSlide = 0;
  carouselWrapperStyle = {}

  next() {
    if (this.currentSlide + 1 === this.items.length) return;
    this.currentSlide = (this.currentSlide + 1) % this.items.length;
    const offset = this.currentSlide * this.itemWidth;
    const myAnimation: AnimationFactory = this.buildAnimation(offset);
    this.player = myAnimation.create(this.carousel.nativeElement);
    this.player.play();
  }

  private buildAnimation(offset) {
    return this.builder.build([
      animate(this.timing, style({ transform: `translateX(-${offset}px)` }))
    ]);
  }

  prev() {
    if (this.currentSlide === 0) return;

    this.currentSlide = ((this.currentSlide - 1) + this.items.length) % this.items.length;
    const offset = this.currentSlide * this.itemWidth;

    const myAnimation: AnimationFactory = this.buildAnimation(offset);
    this.player = myAnimation.create(this.carousel.nativeElement);
    this.player.play();
  }

  constructor(private builder: AnimationBuilder) {
  }

  ngAfterViewInit() {
    // For some reason only here I need to add setTimeout, in my local env it's working without this.
    setTimeout(() => {
      this.itemWidth = this.itemsElements.first.nativeElement.getBoundingClientRect().width;
      this.carouselWrapperStyle = {
        width: `${this.itemWidth}px`
      }
    });

  }

}
